<?php


/**
 * navigationUniversell (aQnavigation)
 *
 * @version $Id: atelierq.navigation.class.inc.php 9572 2009-03-02 17:47:08Z akm $
 * @author Andreas Kummer, w3concepts AG
 * @copyright Copyright &copy; 2009, w3concepts AG
 *
 * History:
 * - 06.09.2023: Murat Purc, adapted to PHP <= 8.1 & PHP >= 7.0, and some rework.
 * - 17.12.2023: Murat Purc, fixed typo getNetLevel() to getNextLevel().
 */
class aQnavigation
{
    /**
     * @var int
     */
    private $startId;
    /**
     * @var int
     */
    private $client;
    /**
     * @var int
     */
    private $lang;
    /**
     * @var int
     */
    private $idcat;
    /**
     * @var string
     */
    private $excludeItemsStartingWith = '__';
    /**
     * @var cDb
     */
    private $db;
    /**
     * @var cAuth
     */
    private $auth;
    /**
     * @var array
     */
    private $cfg;
    /**
     * @var array
     */
    private $before = [];
    /**
     * @var array
     */
    private $beforeEach = [];
    /**
     * @var array
     */
    private $between = [];
    /**
     * @var array
     */
    private $after = [];
    /**
     * @var array
     */
    private $id = [];
    /**
     * @var array
     */
    private $navigationTree = [];
    /**
     * @var array
     */
    private $firstId = [];
    /**
     * @var array
     */
    private $beforeSelected = [];
    /**
     * @var array
     */
    private $maskNormal = [];
    /**
     * @var array
     */
    private $maskNormalSub = [];
    /**
     * @var array
     */
    private $maskSelected = [];
    /**
     * @var array
     */
    private $maskSelectedSub = [];
    /**
     * @var array
     */
    private $maskActive = [];
    /**
     * @var array
     */
    private $maskActiveSub = [];
    /**
     * @var array
     */
    private $sub = [];

    /**
     * Constructor of class.
     *
     * @param int|string $startId Primary key of the start category.
     */
    public function __construct($startId)
    {
        $this->startId = is_int($startId) ? $startId : cSecurity::toInteger($startId);
        $this->client = cSecurity::toInteger(cRegistry::getClientId());
        $this->lang = cSecurity::toInteger(cRegistry::getLanguageId());
        $this->idcat = cSecurity::toInteger($_GET['pretend'] ?? cRegistry::getCategoryId());
        $this->db = cRegistry::getDb();
        $this->auth = cRegistry::getAuth();
        $this->cfg = cRegistry::getConfig();
        $this->setStartIds();
        $this->getNavigation();
    }

    /**
     * Reads the navigation structure from the database.
     */
    private function getNavigation()
    {
        if (!isset($this->id[0]) || $this->startId != $this->id[0]) {
            // If the StartId is not the same as the top level of the currently selected category,
            // then the selected category lies logically outside the tree selected for navigation.
            // In this case, only the main navigation should be shown.
            $this->id[0] = $this->startId;
            for ($i = 1; $i <= 4; $i++) {
                $this->id[$i] = -1;
            }
        }

        if ($this->auth->auth['uid'] == '' || !$this->isNumeric($this->auth->auth['uid'])) {
            // User is not logged in
            $user = 0;
        } else {
            // User is logged in
            $user = $this->auth->auth['uid'];
        }

        $sql = "-- aQnavigation->getNavigation()
            SELECT DISTINCT
                a.idcat,
                a.parentid,
                a.preid,
                a.postid,
                a.parentid,
                b.name,
                b.idlang,
                IF (b.visible = 1 && (b.public = 1 || d.idfrontenduser IS NOT NULL), 1, 0) AS visible
            FROM
                {$this->cfg['tab']['cat']} AS a 
            LEFT JOIN 
                {$this->cfg['tab']['cat_lang']} AS b
                ON a.idcat = b.idcat
            LEFT JOIN
                {$this->cfg['tab']['frontendpermissions']} AS c
                ON (
                    (c.item = b.idcatlang OR c.item = '__GLOBAL__') AND
                    c.idlang = b.idlang AND
                    c.plugin = 'category'
                )
            LEFT JOIN
                {$this->cfg['tab']['frontendgroupmembers']} AS d
                ON c.idfrontendgroup = d.idfrontendgroup AND
                    d.idfrontenduser = {$user}
            WHERE
                a.idclient = {$this->client} AND (
                    a.parentid = {$this->id[0]} OR
                    a.parentid = {$this->id[1]} OR
                    a.parentid = {$this->id[2]} OR
                    a.parentid = {$this->id[3]} OR
                    a.parentid = {$this->id[4]}
                )";

        $this->db->query($sql);
        $this->navigationTree = [];
        $this->firstId = [0, 0, 0, 0, 0];

        while ($this->db->nextRecord()) {
            $_parentId = cSecurity::toInteger($this->db->f('parentid'));
            $_preId = cSecurity::toInteger($this->db->f('preid'));
            $_idLang = cSecurity::toInteger($this->db->f('idlang'));
            $_idCat = cSecurity::toInteger($this->db->f('idcat'));
            for ($i = 0; $i <= 4; $i++) {
                if (($this->firstId[$i] == 0 && $_parentId == $this->id[$i] && $_preId == 0 && $_idLang == $this->lang)
                    || ($this->firstId[$i] == 0 && $_parentId == $this->id[$i] && $_preId == 0 && $_idLang == 0)) {
                    $this->firstId[$i] = $_idCat;
                }
            }
            // It may only be inserted in the navigation tree if no entry already exists or
            // - if one already exists - the new entry corresponds to the currently selected language.
            if (!isset($this->navigationTree[$_parentId][$_idCat]) || $_idLang == $this->lang) {
                $this->navigationTree[$_parentId][$_idCat] = [
                    'preid' => $_preId,
                    'postid' => cSecurity::toInteger($this->db->f('postid')),
                    'visible' => cSecurity::toInteger($this->db->f('visible')),
                    'name' => $this->db->f('name'),
                    'idlang' => $_idLang,
                ];
            }
        }
    }

    /**
     * Output navigation to standard output.
     *
     * @param int $type Type of navigation
     * @param int $next
     * @return string|bool
     */
    public function showNavigation(int $type, int $next = -1)
    {
        static $first;
        static $output = [false, false, false, false, false, false, false];
        $returnValue = "";

        if ($next == 0) {
            return !empty($output[$type]) ? ($this->after[$type] ?? '') : false;
        }

        if ($next == -1) {
            $returnValue .= ($this->before[$type] ?? '') . '#START#';
            $next = $this->firstId[$type] ?? -1;
            $first[$type] = true;
        }

        $parentId = $this->id[$type] ?? -1;
        $nextNavItem = $this->navigationTree[$parentId][$next] ?? [];
        $link = 'front_content.php?idcat=' . $next;

        if (!empty($nextNavItem) && $nextNavItem['visible'] == 1 && $nextNavItem['idlang'] == $this->lang
            && $this->isVisible($nextNavItem['name']))
        {
            if (!$first[$type]) {
                $returnValue .= $this->between[$type] ?? '';
            }
            if (strstr($nextNavItem['name'], '$$$')) {
                $name = explode('$$$', $nextNavItem['name']);
                $name1 = $name[0];
                $name2 = $name[1];
            } else {
                $name1 = $nextNavItem['name'];
                $name2 = '';
            }
            $sub = $this->sub[$type] ?? '';
            if ($this->isActive($next) & !empty($this->maskActive[$type]) & !empty($this->maskActive[$type])) {
                $returnValue .= $this->beforeSelected[$type] ?? '';
                $returnValue .= str_replace('{sub}', $sub, str_replace('{name2}', $name2, str_replace('{name1}', $name1, str_replace('{link}', $link,
                    ($sub == '')
                        ? ($this->maskActive[$type] ?? '')
                        : ($this->maskActiveSub[$type] ?? '')))));
            } elseif ($this->isSelected($next)) {
                $returnValue .= $this->beforeSelected[$type] ?? '';
                $returnValue .= str_replace('{sub}', $sub, str_replace('{name2}', $name2, str_replace('{name1}', $name1, str_replace('{link}', $link,
                    ($sub == '')
                        ? ($this->maskSelected[$type] ?? '')
                        : ($this->maskSelectedSub[$type] ?? '')))));
            } else {
                $returnValue .= $this->beforeEach[$type] ?? '';
                $returnValue .= str_replace('{sub}', $sub, str_replace('{name2}', $name2, str_replace('{name1}', $name1, str_replace('{link}', $link,
                    $this->maskNormal[$type] ?? ''))));
            }
            $first[$type] = false;
            $output[$type] = true;
        } elseif (!$output && (empty($nextNavItem) || $nextNavItem['postid'] == 0)) {
            // The navigation contains nothing
        }

        $returnValue = $returnValue . $this->showNavigation($type, $nextNavItem['postid'] ?? 0);
        $returnValue = preg_replace('/#START#(.*?)\\{iffirst:(.*?)\\}/', "$1$2", $returnValue);
        $returnValue = preg_replace('/\\{iffirst:(.*?)\\}/', "", $returnValue);
        //$returnValue = preg_replace('/\\{iflast:(.*?)\\}(.*?)#END#/', "$1$2", $returnValue);
        $returnValue = preg_replace('/\\{iflast:(.*?)\\}(.*?)/', "$1$2", $returnValue);
        $returnValue = preg_replace('/\\{iflast:(.*?)\\}/', "", $returnValue);
        $returnValue = preg_replace('/#START#/', "", $returnValue);

        return $returnValue;
    }

    /**
     * @param string $start
     * @return void
     */
    public function setExcludeStart(string $start)
    {
        $this->excludeItemsStartingWith = $start;
    }

    /**
     * Checks if a category is visible.
     * A category is considered visible only if the category name does not begin with the prefix
     * defined as the identifier for excluded categories, see {@see aQnavigation::$excludeItemsStartingWith}.
     *
     * @param string $name
     * @return bool
     */
    public function isVisible(string $name): bool
    {
        if (substr($name, 0, strlen($this->excludeItemsStartingWith)) == $this->excludeItemsStartingWith) {
            return false;
        }

        return true;
    }

    /**
     * Output before the navigation.
     *
     * @param int $level Navigation level
     * @param string $text The text/markup
     */
    public function before(int $level, string $text)
    {
        $this->before[$level] = $text;
    }

    /**
     * Output before each entry.
     *
     * @param int $level Navigation level
     * @param string $text The text/markup
     */
    public function beforeEach(int $level, string $text)
    {
        $this->beforeEach[$level] = $text;
    }

    /**
     * Output before each selected menu item.
     *
     * @param int $level Navigation level
     * @param string $text The text/markup
     */
    public function beforeSelected(int $level, string $text)
    {
        $this->beforeSelected[$level] = $text;
    }

    /**
     * Output between two menu items.
     *
     * @param int $level Navigation level
     * @param string $text The text/markup
     */
    public function between(int $level, string $text)
    {
        $this->between[$level] = $text;
    }

    /**
     * Output after the navigation.
     *
     * @param int $level Navigation level
     * @param string $text The text/markup
     */
    public function after(int $level, string $text)
    {
        $this->after[$level] = $text;
    }

    /**
     * @param int $level Navigation level
     * @param string $text The text/markup
     */
    public function maskNormal(int $level, string $text)
    {
        $this->maskNormal[$level] = $text;
    }

    /**
     * @param int $level Navigation level
     * @param string $text The text/markup
     */
    public function maskNormalSub(int $level, string $text)
    {
        $this->maskNormalSub[$level] = $text;
    }

    /**
     * @param int $level Navigation level
     * @param string $text The text/markup
     */
    public function maskSelected(int $level, string $text)
    {
        $this->maskSelected[$level] = $text;
    }

    /**
     * @param int $level Navigation level
     * @param string $text The text/markup
     */
    public function maskSelectedSub(int $level, string $text)
    {
        $this->maskSelectedSub[$level] = $text;
    }

    /**
     * @param int $level Navigation level
     * @param string $text The text/markup
     */
    public function maskActive(int $level, string $text)
    {

        $this->maskActive[$level] = $text;
    }

    /**
     * @param int $level Navigation level
     * @param string $text The text/markup
     */
    public function maskActiveSub(int $level, string $text)
    {
        $this->maskActiveSub[$level] = $text;
    }

    /**
     * @param int $level Navigation level
     * @param string $text The text/markup
     */
    public function setSub(int $level, string $text)
    {
        $this->sub[$level] = $text;
    }

    /**
     * Returns the level of the category.
     *
     * @param int $idcat Primary key of the category.
     * @return int The level of the category.
     * @throws cDbException
     * @throws cInvalidArgumentException
     */
    public function getLevel(int $idcat): int
    {
        if (empty($idcat)) {
            return 0;
        }
        $sql = "-- aQnavigation->getLevel()
            SELECT `level` FROM `%s` WHERE `idcat` = %d
        ";
        $this->db->query($sql, $this->cfg['tab']['cat_tree'], $idcat);
        if ($this->db->nextRecord()) {
            return cSecurity::toInteger($this->db->f('level'));
        } else {
            return 0;
        }
    }

    /**
     * Returns the depth of a category relative to the main menu.
     *
     * @param int $idcat Primary key of the category.
     * @return int Level relative to the main menu
     * @throws cDbException
     * @throws cInvalidArgumentException
     */
    public function getNextLevel(int $idcat): int
    {
        return $this->getLevel($idcat) - $this->getLevel($this->startId);
    }

    /**
     * @deprecated [2023-12-17]
     *     This method is deprecated use method `getNextLevel()`
     */
    public function getNetLevel(int $idcat): int
    {
        cDeprecated('aQnavigation->getNetLevel(): This method is deprecated use method `getNextLevel()`');
        return $this->getNextLevel($idcat);
    }

    /**
     * Determination of the first category entry of each level.
     * This data is stored in the class attribute id.
     */
    private function setStartIds()
    {
        $id = $this->idcat;
        $level = $this->getNextLevel($id);
        $this->id[$level] = $id;
        while ($level > 0) {
            $this->id[$level - 1] = $this->getParentId($id);
            $id = $this->id[$level - 1];
            $level = $this->getNextLevel($id);
        }
        if (empty($this->id)) {
            $this->id[0] = $this->startId;
        }
        for ($i = 1; $i <= 4; $i++) {
            if (empty($this->id[$i])) {
                $this->id[$i] = -1;
            }
        }
        ksort($this->id);
    }

    /**
     * Finding the parent of the category with primary key idcat.
     *
     * @param int $idcat Primary key of the child element whose parent element is to be determined.
     * @return int Primary key of the parent element.
     * @throws cDbException
     * @throws cInvalidArgumentException
     */
    public function getParentId(int $idcat): int
    {
        $sql = "-- aQnavigation->getParentId()
            SELECT `parentid` FROM `%s` WHERE `idcat` = %d
        ";
        $this->db->query($sql, $this->cfg['tab']['cat'], $idcat);
        if ($this->db->nextRecord()) {
            return cSecurity::toInteger($this->db->f('parentid'));
        } else {
            return 0;
        }
    }

    /**
     * Whether a navigation level contains elements or not.
     *
     * @param int $level Level of navigation.
     * @return bool True if the navigation level is empty (contains no items). Otherwise false.
     */
    public function navigationEmpty(int $level): bool
    {
        return empty($this->firstId[$level]);
    }

    /**
     * Whether a category is selected or not. A category is considered selected if it is the
     * current category or a direct or indirect parent of the current category.
     *
     * @param int $idcat Primary key of the category whose status is requested.
     * @return bool True if the requested category is the same as the current category or is
     *      a direct or indirect parent of the current category.
     */
    public function isSelected(int $idcat): bool
    {
        return in_array($idcat, $this->id);
    }

    /**
     * Whether the category is the active category or not.
     *
     * @param int $idcat Primary key of the category whose status is requested.
     * @return bool True if category is active.
     */
    public function isActive(int $idcat): bool
    {
        return $idcat === cSecurity::toInteger(cRegistry::getCategoryId());
    }

    /**
     * Returns true if the passed string consists exclusively of numeric digits.
     *
     * @param string $parameter String parameter to be checked.
     * @return bool True if the passed string consists exclusively of numeric digits.
     */
    public function isNumeric(string $parameter): bool
    {
        return !preg_match("/[^0-9]/", $parameter);
    }

}
